#ifndef _Application_h
#define _Application_h


#include <gtkmm/window.h>
#include <gtkmm/notebook.h>
#include <gtkmm/box.h>

#include "Document.h"
#include "MenuBar.h"
#include "Statusbar.h"
#include "WaveformSystem.h"
#include "Options.h"

class Application : public Gtk::Window
{
public:
	Application(BaseObjectType *cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);//int argc, char *argv[]);
	~Application();

	void init(OptionGroup &options);
protected:

	/*
	 *
	 */
	void on_config_interface_changed(const Glib::ustring &key, const Glib::ustring &value);
	
	/*
	 *
	 */
	void load_config();

	/*
	 *
	 */
	void on_execute_action(const Glib::ustring &name);

	// FILE
	void new_subtitle();
	void open_subtitle();
	void save_subtitle();
	void save_as_subtitle();
	void close_subtitle();
	void quit();

	void open_translation();
	void save_translation();

	void import_text();
	void export_text();

	// EDIT
	void select_previous_subtitle();
	void select_next_subtitle();
	void select_all_subtitle();
	void toggle_selection_subtitle();

	void delete_subtitle();
	
	void insert_before();
	void insert_after();
	
	void move_subtitles();
	void add_msecs();
	void remove_msecs();
	void add_msecs_to_start();
	void remove_msecs_to_start();
	void add_msecs_to_duration();
	void remove_msecs_to_duration();

	void apply_translation();

	// TOOLS
	void find_and_replace();
	void check_errors();
	void spell_check();
	void style_editor();
	void split_document();
	void join_document();
	void change_framerate();
	void scale();

	// another
	void open_movie_for_external_video_player();
	void play_movie_with_external_video_player();
	// 
	void about();


	bool ask_to_save_on_exit();
protected:
	
	/*
	 *
	 */
	virtual bool on_delete_event(GdkEventAny *ev);

	/*
	 *	il y a la création d'un nouveau document
	 *	on l'ajoute dans le notebook
	 *	signal emit par DocumentSystem::signal_document_create
	 */
	void on_document_create(Document *doc);

	/*
	 *	on efface le document du notebook
	 *	signal emit par DocumentSystem::signal_document_delete
	 */
	void on_document_delete(Document *doc);

	/*
	 *
	 */
	void on_current_document_changed(Document *doc);

	/*
	 *	Changement dans le notebook de la page editer
	 *	On recupere la page pusi on init DocumentSystem avec le document
	 */
	void on_signal_switch_page(GtkNotebookPage* page, guint page_num);

	/*
	 *
	 */
	void on_close_document(Document *doc);

	/*
	 *	mise a jour d'info (filename, tooltip, ...) dans la page
	 */
	void update_document_property(Document *doc);

	/*
	 *	retourne le widget (notebook) par rapport au document
	 */
	Gtk::Widget* get_widget(Document *doc);

	/*
	 *
	 */
	void set_display_video_player(bool state);
	void set_display_waveform(bool state);

	void connect_document(Document *doc);
	void disconnect_document(Document *doc);


	/*
	 *	sauvegarde les documents toute les "autosave-minutes" (Config)
	 */
	bool on_autosave_files();
protected:
	Gtk::VBox*			m_vboxMain;
	MenuBar					m_menubar;
	VideoPlayer*		m_videoPlayer;
	WaveformSystem*	m_waveform_system;
	Gtk::Notebook*	m_notebook_documents;
	Statusbar*			m_statusbar;

	std::list<sigc::connection> m_document_connections;

	// uri for external video player
	Glib::ustring		m_uri_movie_external_video_player;
	//
	sigc::connection	m_autosave_timeout;
};

#endif//_Application_h

